from r3frame.globs import pg

class Process:
    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name

    def callback(self, data) -> bool: raise NotImplementedError
    def fallback(self, data) -> bool: raise NotImplementedError



from r3frame.globs import pg

class Tooltip:
    def __init__(self, font_path: str, text: str="Tooltip", size: list[int|float]=[100, 64]) -> None:
        self.show_border = False
        
        self.size = size
        self.location = [0, 0]
        self.image = pg.Surface(size)

        self.offset = [0, 0]
        self.padding: list[int] = [0, 0]
        self.color: list[int] = [0, 0, 0]

        self.border_size = [1, 1]
        self.border_radius = [0, 0, 0, 0]
        self.border_color = [255, 255, 255]

        self.text = text
        self.text_size: int = 18
        self.text_color: list[int] = [255, 255, 255]

        self.font_path = font_path
        self.font: pg.Font = pg.Font(font_path, self.text_size)

    def border(self) -> pg.Rect:
        return pg.Rect(
            [(self.location[0] + self.offset[0]) - self.border_size[0],
             (self.location[1] + self.offset[1]) - self.border_size[1],], 
            [self.size[0] + self.border_size[0],
             self.size[1] + self.border_size[1],]
        )

    def render(self, surface: pg.Surface) -> None: 
        self.image.fill(self.color)
        self.image.blit(
        self.font.render(self.text, True, self.text_color, self.color), [self.padding[0], self.padding[1]])
        surface.blit(self.image, [
            self.location[0] + self.offset[0],
            self.location[1] + self.offset[1]
        ])
        if self.show_border:
            pg.draw.rect(surface, self.border_color, self.border(), width=self.border_size[0],
                border_top_left_radius=self.border_radius[0],
                border_top_right_radius=self.border_radius[1],
                border_bottom_left_radius=self.border_radius[2],
                border_bottom_right_radius=self.border_radius[3]
            )


from r3frame.globs import pg
from r3frame.util import _asset_path

from r3frame.app.scene import Scene
from r3frame.app.event import EventManager
from r3frame.app.input import Keyboard, Mouse
from r3frame.app.rsrc import Clock, Window, Camera, Renderer, AssetManager

class Application:
    def __init__(self, name: str="My App", window_size: list[int]=[800, 600]) -> None:
        self.name = name
        self.clock = Clock()
        self.assets = AssetManager()
        self.events = EventManager()

        self.scene: str = None
        self.scene: Scene = None
        self.scenes: dict[str, Scene] = {}

        self.window = Window(window_size, window_size)
        self.window.title = name
        self.window.icon = pg.image.load(_asset_path("images/r3-logo.ico"))
        self.window.configure()

        self.camera = Camera(self.window)
        self.renderer = Renderer(self.camera)

        self.configure()

    def set_scene(self, scene: Scene) -> None:
        self.scenes[scene.name] = scene
        self.scene = self.scenes[self.scene]
    
    def rem_scene(self, key: str) -> Scene|None:
        if self.get_scene(key) is not None:
            self.scenes.pop(key, None)
            self.scene = None
    
    def get_scene(self, key: str) -> Scene|None:
        return self.scenes.get(key, None)

    def configure(self) -> None: raise NotImplementedError
    def handle_events(self) -> None: raise NotImplementedError
    def handle_update(self) -> None: raise NotImplementedError
    def handle_render(self) -> None: raise NotImplementedError

    def exit(self) -> None: raise NotImplementedError
    def run(self) -> None:
        while not self.events.quit:
            self.clock.update()
            self.events.update()

            if isinstance(self.scene, Scene):
                self.scene.handle_events()
                self.handle_events()

                self.scene.handle_update()
                self.scene.interface.update(self.events)
                self.handle_update()
                self.camera.update(self.clock.delta)

                self.scene.handle_render()
                self.scene.interface.render()
                self.handle_render()
            else:
                self.handle_events()

                self.handle_update()
                self.camera.update(self.clock.delta)

                self.handle_render()

            self.renderer.render()
            self.window.update()
            self.clock.rest()
        else:
            self.exit()



import r3frame.app as app
import r3frame.app.ui as ui
import r3frame.game as game
import r3frame.version as version
from r3frame.util import sine_wave_value, damp_exp, damp_lin, point_inside, dist_to, angle_to

from r3frame.globs import os
if "R3FRAME_NO_PROMT" not in os.environ:
    print(
        f"r3frame {version.R3FRAME_YEAR}.{version.R3FRAME_MINOR}.{version.R3FRAME_PATCH} | [ random quote here ]"
    )